@@ -1,44 +1,46 @@
-/*-------------------------------------------------------------------------------------*/
-/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.8.0      */
-/*                                                                                     */
-/*                                                                                     */
-/*  NOMAD - version 3.8.0 has been created by                                          */
-/*                 Charles Audet        - Ecole Polytechnique de Montreal              */
-/*                 Sebastien Le Digabel - Ecole Polytechnique de Montreal              */
-/*                 Christophe Tribes    - Ecole Polytechnique de Montreal              */
-/*                                                                                     */
-/*  The copyright of NOMAD - version 3.8.0 is owned by                                 */
-/*                 Sebastien Le Digabel - Ecole Polytechnique de Montreal              */
-/*                 Christophe Tribes    - Ecole Polytechnique de Montreal              */
-/*                                                                                     */
-/*  NOMAD v3 has been funded by AFOSR and Exxon Mobil.                                 */
-/*                                                                                     */
-/*  NOMAD v3 is a new version of NOMAD v1 and v2. NOMAD v1 and v2 were created and     */
-/*  developed by Mark Abramson, Charles Audet, Gilles Couture and John E. Dennis Jr.,  */
-/*  and were funded by AFOSR and Exxon Mobil.                                          */
-/*                                                                                     */
-/*                                                                                     */
-/*  Contact information:                                                               */
-/*    Ecole Polytechnique de Montreal - GERAD                                          */
-/*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada                  */
-/*    e-mail: nomad@gerad.ca                                                           */
-/*    phone : 1-514-340-6053 #6928                                                     */
-/*    fax   : 1-514-340-5665                                                           */
-/*                                                                                     */
-/*  This program is free software: you can redistribute it and/or modify it under the  */
-/*  terms of the GNU Lesser General Public License as published by the Free Software   */
-/*  Foundation, either version 3 of the License, or (at your option) any later         */
-/*  version.                                                                           */
-/*                                                                                     */
-/*  This program is distributed in the hope that it will be useful, but WITHOUT ANY    */
-/*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A    */
-/*  PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.   */
-/*                                                                                     */
-/*  You should have received a copy of the GNU Lesser General Public License along     */
-/*  with this program. If not, see <http://www.gnu.org/licenses/>.                     */
-/*                                                                                     */
-/*  You can find information on the NOMAD software at www.gerad.ca/nomad               */
-/*-------------------------------------------------------------------------------------*/
+/*---------------------------------------------------------------------------------*/
+/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search -                */
+/*                                                                                 */
+/*  NOMAD - version 3.9.1 has been created by                                      */
+/*                 Charles Audet               - Ecole Polytechnique de Montreal   */
+/*                 Sebastien Le Digabel        - Ecole Polytechnique de Montreal   */
+/*                 Viviane Rochon Montplaisir - Ecole Polytechnique de Montreal   */
+/*                 Christophe Tribes           - Ecole Polytechnique de Montreal   */
+/*                                                                                 */
+/*  The copyright of NOMAD - version 3.9.1 is owned by                             */
+/*                 Sebastien Le Digabel        - Ecole Polytechnique de Montreal   */
+/*                 Viviane Rochon Montplaisir - Ecole Polytechnique de Montreal   */
+/*                 Christophe Tribes           - Ecole Polytechnique de Montreal   */
+/*                                                                                 */
+/*  NOMAD v3 has been funded by AFOSR and Exxon Mobil.                             */
+/*                                                                                 */
+/*  NOMAD v3 is a new version of NOMAD v1 and v2. NOMAD v1 and v2 were created     */
+/*  and developed by Mark Abramson, Charles Audet, Gilles Couture, and John E.     */
+/*  Dennis Jr., and were funded by AFOSR and Exxon Mobil.                          */
+/*                                                                                 */
+/*  Contact information:                                                           */
+/*    Ecole Polytechnique de Montreal - GERAD                                      */
+/*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada              */
+/*    e-mail: nomad@gerad.ca                                                       */
+/*    phone : 1-514-340-6053 #6928                                                 */
+/*    fax   : 1-514-340-5665                                                       */
+/*                                                                                 */
+/*  This program is free software: you can redistribute it and/or modify it        */
+/*  under the terms of the GNU Lesser General Public License as published by       */
+/*  the Free Software Foundation, either version 3 of the License, or (at your     */
+/*  option) any later version.                                                     */
+/*                                                                                 */
+/*  This program is distributed in the hope that it will be useful, but WITHOUT    */
+/*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or          */
+/*  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License    */
+/*  for more details.                                                              */
+/*                                                                                 */
+/*  You should have received a copy of the GNU Lesser General Public License       */
+/*  along with this program. If not, see <http://www.gnu.org/licenses/>.           */
+/*                                                                                 */
+/*  You can find information on the NOMAD software at www.gerad.ca/nomad           */
+/*---------------------------------------------------------------------------------*/
+
 /**
   \file   Parameter_Entry.hpp
   \brief  Parameter entry (headers)
@@ -57,9 +59,9 @@ namespace NOMAD {
   /// Parameter entry.
   /**
      - Describes the data relative to a parameter in a parameters file.
-     - Objets of this class are stored in a NOMAD::Parameter_Entries object.
+     - Objects of this class are stored in a NOMAD::Parameter_Entries object.
   */
-  class Parameter_Entry : private NOMAD::Uncopyable {
+  class DLL_API Parameter_Entry : private NOMAD::Uncopyable {
 
   private:
 
@@ -68,6 +70,9 @@ namespace NOMAD {
     bool                   _ok;     ///< If the parameter is valid.
     bool                   _unique; ///< If the parameter is unique.
     Parameter_Entry      * _next;   ///< Acces to the next parameter.
+      
+    std::string            _param_file; ///< File from which this parameter was read
+    int                    _line;       ///< Line for this parameter in _param_file
 
     /// If the parameter has been interpreted.
     bool _has_been_interpreted; 
@@ -80,7 +85,7 @@ namespace NOMAD {
        \param entry           A string describing the parameter entry -- \b IN.
        \param remove_comments A boolean equal to \c true if entries after
                               \c '#' are ignored -- \b IN
-			      -- \b optional (default = \c true).
+                  -- \b optional (default = \c true).
     */
     Parameter_Entry ( const std::string & entry , bool remove_comments = true );
 
@@ -140,6 +145,18 @@ namespace NOMAD {
     */
     bool has_been_interpreted ( void ) const { return _has_been_interpreted; }
 
+    /// Access to the parameter file of the parameter.
+    /**
+       \return The parameter file where this parameter was read.
+    */
+    const std::string & get_param_file ( void ) const { return _param_file; }
+
+    /// Access to the line number for this parameter in the parameter file.
+    /**
+       \return The line number at which this parameter can be found in the parameter file.
+    */
+    const int & get_line ( void ) const { return _line; }
+
     /*---------------*/
     /*  SET methods  */
     /*---------------*/
@@ -158,6 +175,12 @@ namespace NOMAD {
 
     /// Set the \c _has_been_interpreted flag. to \c true.
     void set_has_been_interpreted  ( void ) { _has_been_interpreted = true; }
+
+    /// Set the name of the parameter file \c _param_file
+    void set_param_file( const std::string param_file ) { _param_file = param_file; }
+    
+    /// Set the line \c _line for this parameter in the parameter file
+    void set_line( int line ) { _line = line; }
     
     /// Comparison with another entry.
     /**
@@ -175,7 +198,7 @@ namespace NOMAD {
   };
 
   /// Allows the comparison of two NOMAD::Parameter_Entry objects.
-  struct Parameter_Entry_Comp {
+  struct DLL_API Parameter_Entry_Comp {
     /// Comparison of two NOMAD::Parameter_Entry objects.
     /**
        \param  p1 Pointer to the first NOMAD::Parameter_Entry  -- \b IN.
@@ -195,7 +218,7 @@ namespace NOMAD {
      \return    The NOMAD::Display object.
   */
   inline const NOMAD::Display & operator << ( const NOMAD::Display         & out ,
-					      const NOMAD::Parameter_Entry & e     ) 
+                          const NOMAD::Parameter_Entry & e     ) 
   {
     e.display ( out );
     return out;
