@@ -1,44 +1,46 @@
-/*-------------------------------------------------------------------------------------*/
-/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.8.0      */
-/*                                                                                     */
-/*                                                                                     */
-/*  NOMAD - version 3.8.0 has been created by                                          */
-/*                 Charles Audet        - Ecole Polytechnique de Montreal              */
-/*                 Sebastien Le Digabel - Ecole Polytechnique de Montreal              */
-/*                 Christophe Tribes    - Ecole Polytechnique de Montreal              */
-/*                                                                                     */
-/*  The copyright of NOMAD - version 3.8.0 is owned by                                 */
-/*                 Sebastien Le Digabel - Ecole Polytechnique de Montreal              */
-/*                 Christophe Tribes    - Ecole Polytechnique de Montreal              */
-/*                                                                                     */
-/*  NOMAD v3 has been funded by AFOSR and Exxon Mobil.                                 */
-/*                                                                                     */
-/*  NOMAD v3 is a new version of NOMAD v1 and v2. NOMAD v1 and v2 were created and     */
-/*  developed by Mark Abramson, Charles Audet, Gilles Couture and John E. Dennis Jr.,  */
-/*  and were funded by AFOSR and Exxon Mobil.                                          */
-/*                                                                                     */
-/*                                                                                     */
-/*  Contact information:                                                               */
-/*    Ecole Polytechnique de Montreal - GERAD                                          */
-/*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada                  */
-/*    e-mail: nomad@gerad.ca                                                           */
-/*    phone : 1-514-340-6053 #6928                                                     */
-/*    fax   : 1-514-340-5665                                                           */
-/*                                                                                     */
-/*  This program is free software: you can redistribute it and/or modify it under the  */
-/*  terms of the GNU Lesser General Public License as published by the Free Software   */
-/*  Foundation, either version 3 of the License, or (at your option) any later         */
-/*  version.                                                                           */
-/*                                                                                     */
-/*  This program is distributed in the hope that it will be useful, but WITHOUT ANY    */
-/*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A    */
-/*  PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.   */
-/*                                                                                     */
-/*  You should have received a copy of the GNU Lesser General Public License along     */
-/*  with this program. If not, see <http://www.gnu.org/licenses/>.                     */
-/*                                                                                     */
-/*  You can find information on the NOMAD software at www.gerad.ca/nomad               */
-/*-------------------------------------------------------------------------------------*/
+/*---------------------------------------------------------------------------------*/
+/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search -                */
+/*                                                                                 */
+/*  NOMAD - version 3.9.1 has been created by                                      */
+/*                 Charles Audet               - Ecole Polytechnique de Montreal   */
+/*                 Sebastien Le Digabel        - Ecole Polytechnique de Montreal   */
+/*                 Viviane Rochon Montplaisir - Ecole Polytechnique de Montreal   */
+/*                 Christophe Tribes           - Ecole Polytechnique de Montreal   */
+/*                                                                                 */
+/*  The copyright of NOMAD - version 3.9.1 is owned by                             */
+/*                 Sebastien Le Digabel        - Ecole Polytechnique de Montreal   */
+/*                 Viviane Rochon Montplaisir - Ecole Polytechnique de Montreal   */
+/*                 Christophe Tribes           - Ecole Polytechnique de Montreal   */
+/*                                                                                 */
+/*  NOMAD v3 has been funded by AFOSR and Exxon Mobil.                             */
+/*                                                                                 */
+/*  NOMAD v3 is a new version of NOMAD v1 and v2. NOMAD v1 and v2 were created     */
+/*  and developed by Mark Abramson, Charles Audet, Gilles Couture, and John E.     */
+/*  Dennis Jr., and were funded by AFOSR and Exxon Mobil.                          */
+/*                                                                                 */
+/*  Contact information:                                                           */
+/*    Ecole Polytechnique de Montreal - GERAD                                      */
+/*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada              */
+/*    e-mail: nomad@gerad.ca                                                       */
+/*    phone : 1-514-340-6053 #6928                                                 */
+/*    fax   : 1-514-340-5665                                                       */
+/*                                                                                 */
+/*  This program is free software: you can redistribute it and/or modify it        */
+/*  under the terms of the GNU Lesser General Public License as published by       */
+/*  the Free Software Foundation, either version 3 of the License, or (at your     */
+/*  option) any later version.                                                     */
+/*                                                                                 */
+/*  This program is distributed in the hope that it will be useful, but WITHOUT    */
+/*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or          */
+/*  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License    */
+/*  for more details.                                                              */
+/*                                                                                 */
+/*  You should have received a copy of the GNU Lesser General Public License       */
+/*  along with this program. If not, see <http://www.gnu.org/licenses/>.           */
+/*                                                                                 */
+/*  You can find information on the NOMAD software at www.gerad.ca/nomad           */
+/*---------------------------------------------------------------------------------*/
+
 /**
  \file   Point.hpp
  \brief  Custom class for points (headers)
@@ -48,6 +50,7 @@
  */
 
 // CASE Visual Studio C++ compiler
+// Silent warning about std::inner_product
 #ifdef _MSC_VER
 #pragma warning(disable:4996)
 #endif
@@ -93,7 +96,7 @@ namespace NOMAD {
     /**
      Coordinates are NOMAD::Double objects.
      */
-    class Point {
+    class DLL_API Point {
         
     private:
         
@@ -232,6 +235,14 @@ namespace NOMAD {
          \return The \c (i+1)th coordinate.
          */
         const NOMAD::Double & operator [] ( int i ) const;
+
+        /// Const operator \c [].
+        /**
+         \param i The index (0 for the first element) -- \b IN.
+         \return The \c (i+1)th coordinate.
+         */
+        const NOMAD::Double & operator [] ( size_t i ) const { return this->operator[](static_cast<int>(i));}
+
         
         /// Non-const operator \c [].
         /**
@@ -239,6 +250,15 @@ namespace NOMAD {
          \return The \c (i+1)th coordinate.
          */
         NOMAD::Double & operator [] ( int i );
+
+        
+        /// Non-const operator \c [].
+        /**
+         \param i The index (0 for the first element) -- \b IN.
+         \return The \c (i+1)th coordinate.
+         */
+        NOMAD::Double & operator [] ( size_t i ){ return this->operator[](static_cast<int>(i));}
+
         
         /// Size of the point in memory.
         /**
@@ -313,7 +333,7 @@ namespace NOMAD {
 
         /// Squared norm of the point.
         /**
-         \return A NOMAD::Double with the squared norm of the point.
+         \return A NOMAD::Double with the infinite norm of the point.
          */
         const NOMAD::Double infinite_norm ( void ) const
         {
@@ -350,6 +370,13 @@ namespace NOMAD {
          */
         const NOMAD::Double get_angle ( const Point & x ) const;
         
+        /// Angle of direciton (given as a point) with a list of directions (given as a list of directions) \c x.
+        /**
+         \param list_x The other point \c x -- \b IN.
+         \return The angle between \c *this and \c list_x.
+         */
+        const NOMAD::Double get_angle ( const std::list<NOMAD::Point> & list_x ) const ;
+        
         /// Mutiplication with a scalar.
         /**
          - This implements \c *this \c = \c d \c * \c *this.
