@@ -1,29 +1,12 @@
 /*-------------------------------------------------------------------------------------*/
-/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.8.0      */
+/*  sgtelib - A surrogate model library for derivative-free optimization               */
+/*  Version 2.0.2                                                                      */
 /*                                                                                     */
+/*  Copyright (C) 2012-2016  Sebastien Le Digabel - Ecole Polytechnique, Montreal      */ 
+/*                           Bastien Talgorn - McGill University, Montreal             */
 /*                                                                                     */
-/*  NOMAD - version 3.8.0 has been created by                                          */
-/*                 Charles Audet        - Ecole Polytechnique de Montreal              */
-/*                 Sebastien Le Digabel - Ecole Polytechnique de Montreal              */
-/*                 Christophe Tribes    - Ecole Polytechnique de Montreal              */
-/*                                                                                     */
-/*  The copyright of NOMAD - version 3.8.0 is owned by                                 */
-/*                 Sebastien Le Digabel - Ecole Polytechnique de Montreal              */
-/*                 Christophe Tribes    - Ecole Polytechnique de Montreal              */
-/*                                                                                     */
-/*  NOMAD v3 has been funded by AFOSR and Exxon Mobil.                                 */
-/*                                                                                     */
-/*  NOMAD v3 is a new version of NOMAD v1 and v2. NOMAD v1 and v2 were created and     */
-/*  developed by Mark Abramson, Charles Audet, Gilles Couture and John E. Dennis Jr.,  */
-/*  and were funded by AFOSR and Exxon Mobil.                                          */
-/*                                                                                     */
-/*                                                                                     */
-/*  Contact information:                                                               */
-/*    Ecole Polytechnique de Montreal - GERAD                                          */
-/*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada                  */
-/*    e-mail: nomad@gerad.ca                                                           */
-/*    phone : 1-514-340-6053 #6928                                                     */
-/*    fax   : 1-514-340-5665                                                           */
+/*  Author: Bastien Talgorn                                                            */
+/*  email: bastientalgorn@fastmail.com                                                 */
 /*                                                                                     */
 /*  This program is free software: you can redistribute it and/or modify it under the  */
 /*  terms of the GNU Lesser General Public License as published by the Free Software   */
@@ -37,7 +20,7 @@
 /*  You should have received a copy of the GNU Lesser General Public License along     */
 /*  with this program. If not, see <http://www.gnu.org/licenses/>.                     */
 /*                                                                                     */
-/*  You can find information on the NOMAD software at www.gerad.ca/nomad               */
+/*  You can find information on sgtelib at https://github.com/bastientalgorn/sgtelib   */
 /*-------------------------------------------------------------------------------------*/
 /**
  \file   Sgtelib_Model_Search.cpp
@@ -90,7 +73,7 @@ void NOMAD::Sgtelib_Model_Search::clear_pts ( std::vector<NOMAD::Eval_Point *> &
 /*                                                                  */
 /*  . MODEL_SEARCH_OPTIMISTIC: if true, the direction from the      */
 /*                             model center to the trial point      */
-/*                             is computed and p ossibly used       */
+/*                             is computed and possibly  used       */
 /*                             in the speculative search            */
 /*                             default=yes                          */
 /*                                                                  */
@@ -117,7 +100,6 @@ void NOMAD::Sgtelib_Model_Search::search ( NOMAD::Mads              & mads
     count_search  = false;
     const NOMAD::Display    & out = _p.out();
     const NOMAD::dd_type display_degree = out.get_search_dd();
-    //const bool bool_display = ( string_find(_p.get_SGTELIB_MODEL_DISPLAY(),"S") or (display_degree == NOMAD::FULL_DISPLAY) );
     const bool bool_display = ( _p.get_SGTELIB_MODEL_DISPLAY().size() || (display_degree == NOMAD::FULL_DISPLAY) );
     
     // initial displays:
@@ -237,10 +219,10 @@ void NOMAD::Sgtelib_Model_Search::search ( NOMAD::Mads              & mads
     // Get mads & ev_ctrl
     NOMAD::Stats & stats = mads.get_stats();
     NOMAD::Evaluator_Control * ev_control = _sgtelib_model_manager->get_evaluator_control();
+
     
     const int kkmax = _p.get_SGTELIB_MODEL_TRIALS();
     
-    // MULTIPLE_SEARCH
     for (int kk=0 ; kk<kkmax ; kk++)
     {
         
@@ -355,6 +337,7 @@ void NOMAD::Sgtelib_Model_Search::search ( NOMAD::Mads              & mads
             delete evaluated_pts;
             
             cache_hit = (cache_hits<stats.get_cache_hits());
+            
             // update stats:
             _one_search_stats.add_MS_bb_eval    ( stats.get_bb_eval   () - bbe        );
             _one_search_stats.add_MS_sgte_eval  ( stats.get_sgte_eval () - sgte_eval  );
@@ -734,11 +717,7 @@ bool NOMAD::Sgtelib_Model_Search::filter_cache (  const NOMAD::Display & out
     if (nb_methods==0)
         throw NOMAD::Exception ( "Sgtelib_Model_Search.cpp" ,
                                  __LINE__ ,"method index non valid" );
-    
-    
-    
-    
-    
+
     
     int failure = 0;
     int ni, nmax, iselect;
@@ -830,7 +809,7 @@ bool NOMAD::Sgtelib_Model_Search::filter_cache (  const NOMAD::Display & out
                 dmax = 0;
                 for (i=0 ; i<pp ; i++)
                 {
-                    if ( ( ! keep[i]) && (DTX[i]>=dmax) )
+                    if ( (!keep[i]) && (DTX[i]>=dmax) )
                     {
                         dmax = DTX[i];
                         iselect = i;
@@ -912,7 +891,7 @@ bool NOMAD::Sgtelib_Model_Search::filter_cache (  const NOMAD::Display & out
                 for (i=0 ; i<pp ; i++)
                 {
                     
-                    if ( ( ! keep[i]) && (d_isolation[i]>0) )
+                    if ( (!keep[i]) && (d_isolation[i]>0) )
                     {
                         ni = n_isolation[i];
                         // If criteria undef, then compute.
@@ -944,7 +923,7 @@ bool NOMAD::Sgtelib_Model_Search::filter_cache (  const NOMAD::Display & out
                 nmax = 0;
                 for (i=0 ; i<pp ; i++)
                 {
-                    if ( ( ! keep[i]) && (DTX[i]>0) )
+                    if ( (!keep[i]) && (DTX[i]>0) )
                     {
                         ni = n_density[i];
                         // If criteria undef, then compute.
@@ -1124,11 +1103,6 @@ bool NOMAD::Sgtelib_Model_Search::check_oracle_point  ( const NOMAD::Cache   & c
 } // end check_oracle_points
 
 
-
-
-
-
-
 /*--------------------------------------------------------*/
 /*  insert a trial point in the evaluator control object  */
 /*  (private)                                             */
@@ -1268,10 +1242,6 @@ bool NOMAD::Sgtelib_Model_Search::optimize_model ( const NOMAD::Cache      & cac
     model_param.set_INITIAL_POLL_SIZE ( init_Delta , false );
     model_param.set_INITIAL_MESH_SIZE ( init_delta , false );
     
-    
-    
-    //  model_param.set_INITIAL_MESH_INDEX ( 0 );
-    
     // searches:
     model_param.set_LH_SEARCH ( int(_p.get_SGTELIB_MODEL_EVAL_NB()*0.3) , 0 );
     model_param.set_OPPORTUNISTIC_LH ( false );
@@ -1297,11 +1267,6 @@ bool NOMAD::Sgtelib_Model_Search::optimize_model ( const NOMAD::Cache      & cac
     model_param.set_LOWER_BOUND ( _sgtelib_model_manager->get_extended_lb() );
     model_param.set_UPPER_BOUND ( _sgtelib_model_manager->get_extended_ub() );
     
-    //out << "Bounds : " << std::endl;
-    //out << _sgtelib_model_manager->get_extended_lb() << std::endl;
-    //out << _sgtelib_model_manager->get_extended_ub() << std::endl;
-    
-    
     // Max eval
     model_param.set_MAX_BB_EVAL ( _p.get_SGTELIB_MODEL_EVAL_NB() );
     
@@ -1427,12 +1392,12 @@ bool NOMAD::Sgtelib_Model_Search::optimize_model ( const NOMAD::Cache      & cac
     const int nb_candidates = (i<=0) ? _p.get_bb_max_block_size() : i;
     
     
-    if ( nb_candidates==1 )
+    if (nb_candidates==1)
     {
-        if ( xf )
-            oracle_pts.push_back(xf);
-        else if ( xi )
-            oracle_pts.push_back(xi);
+        NOMAD::Eval_Point * x_add = NULL;
+        if (xf)      x_add = new NOMAD::Eval_Point(*xf,_p.get_bb_nb_outputs());
+        else if (xi) x_add = new NOMAD::Eval_Point(*xi,_p.get_bb_nb_outputs());
+        oracle_pts.push_back(x_add);
     }
     else
     {
@@ -1504,10 +1469,6 @@ bool NOMAD::Sgtelib_Model_Search::optimize_model ( const NOMAD::Cache      & cac
 }
 
 
-
-
-
-
 /*------------------------------------------------------*/
 /*  get best projection                                 */
 /*------------------------------------------------------*/
@@ -1609,8 +1570,6 @@ void NOMAD::Sgtelib_Model_Search::get_best_projection ( const NOMAD::Cache   & c
             set_index.insert(i);
     }
     
-    
-    
     // Build the set of neighboors
     //----------------------------
     
@@ -1922,8 +1881,3 @@ void NOMAD::Sgtelib_Model_Search::get_best_projection ( const NOMAD::Cache   & c
     
 }//
 
-
-
-
-
-// #endif
