@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------------------*/
 /*  sgtelib - A surrogate model library for derivative-free optimization               */
-/*  Version 2.0.1                                                                      */
+/*  Version 2.0.2                                                                      */
 /*                                                                                     */
 /*  Copyright (C) 2012-2017  Sebastien Le Digabel - Ecole Polytechnique, Montreal      */ 
 /*                           Bastien Talgorn - McGill University, Montreal             */
@@ -42,11 +42,11 @@ SGTELIB::Matrix::Matrix ( const std::string & name ,
 
   int i , j;
 
-  X0 = new double * [_nbRows];
+  X_X = new double * [_nbRows];
   for ( i = 0 ; i < _nbRows ; ++i ) {
-    X0[i] = new double [_nbCols];
+    X_X[i] = new double [_nbCols];
     for ( j = 0 ; j < _nbCols ; ++j )
-      X0[i][j] = 0.0;
+      X_X[i][j] = 0.0;
   }
 }//
 
@@ -68,11 +68,11 @@ SGTELIB::Matrix::Matrix ( const std::string & name ,
 
   int i , j;
 
-  X0 = new double * [_nbRows];
+  X_X = new double * [_nbRows];
   for ( i = 0 ; i < _nbRows ; ++i ) {
-    X0[i] = new double [_nbCols];
+    X_X[i] = new double [_nbCols];
     for ( j = 0 ; j < _nbCols ; ++j )
-      X0[i][j] = A[i][j];
+      X_X[i][j] = A[i][j];
   }
 }//
 
@@ -83,7 +83,7 @@ SGTELIB::Matrix::Matrix ( const std::string & file_name ) :
                   _name ( "no_name" ) ,
                   _nbRows    ( 0         ) ,
                   _nbCols    ( 0         ) ,
-                  X0    ( NULL      )   {
+                  X_X    ( NULL      )   {
   *this = import_data(file_name);
 }//
 
@@ -95,9 +95,25 @@ SGTELIB::Matrix::Matrix (void) :
                _name ( "" ) ,
                _nbRows    ( 0   ) ,
                _nbCols    ( 0   ) {
-  X0 = new double * [0];
+  X_X = new double * [0];
 }//
 
+/*---------------------------*/
+/*        constructor 5      */
+/*---------------------------*/
+SGTELIB::Matrix::Matrix (double v) : 
+               _name ( "double" ) ,
+               _nbRows    ( 1   ) ,
+               _nbCols    ( 1   ) {
+  #ifdef SGTELIB_DEBUG
+    SGTELIB::rout << "Matrix Constructor 5\n";
+  #endif
+  X_X = new double * [1];
+  X_X[0] = new double [1];
+  X_X[0][0] = v;
+}//
+
+
 /*---------------------------*/
 /*      copy constructor     */
 /*---------------------------*/
@@ -106,11 +122,11 @@ SGTELIB::Matrix::Matrix ( const SGTELIB::Matrix & A ) :
                           _nbRows    ( A._nbRows    ) ,
                           _nbCols    ( A._nbCols    ) {
   int i , j;
-  X0 = new double * [_nbRows];
+  X_X = new double * [_nbRows];
   for ( i = 0 ; i < _nbRows ; ++i ) {
-    X0[i] = new double [_nbCols];
+    X_X[i] = new double [_nbCols];
     for ( j = 0 ; j < _nbCols ; ++j )
-      X0[i][j] = A.X0[i][j];
+      X_X[i][j] = A.X_X[i][j];
   }
 }//
 
@@ -130,23 +146,23 @@ SGTELIB::Matrix & SGTELIB::Matrix::operator = ( const SGTELIB::Matrix & A ) {
   if ( _nbRows != A._nbRows || _nbCols != A._nbCols ) {
 
     for ( i = 0 ; i < _nbRows ; ++i )
-      delete [] X0[i];
-    delete [] X0;
+      delete [] X_X[i];
+    delete [] X_X;
 
     _nbRows = A._nbRows;
     _nbCols = A._nbCols;
 
-    X0 = new double * [_nbRows];
+    X_X = new double * [_nbRows];
     for ( i = 0 ; i < _nbRows ; ++i ) {
-      X0[i] = new double [_nbCols];
+      X_X[i] = new double [_nbCols];
       for ( j = 0 ; j < _nbCols ; ++j )
-        X0[i][j] = A.X0[i][j];
+        X_X[i][j] = A.X_X[i][j];
     }
   }
   else {
     for ( i = 0 ; i < _nbRows ; ++i )
       for ( j = 0 ; j < _nbCols ; ++j )
-        X0[i][j] = A.X0[i][j];
+        X_X[i][j] = A.X_X[i][j];
   }
     
   _name = A._name;
@@ -244,7 +260,7 @@ SGTELIB::Matrix SGTELIB::Matrix::string_to_row  ( const std::string & s , int nb
   double v;
   std::stringstream ss( s );
   int i=0;
-  while( ss >> v ) row.X0[0][i++] = v;
+  while( ss >> v ) row.X_X[0][i++] = v;
   if (i++!=nbCols){
     SGTELIB::rout << "In line \"" << s << "\"\n";
     SGTELIB::rout << "Found " << i << " components\n";
@@ -326,11 +342,21 @@ SGTELIB::Matrix operator / (const SGTELIB::Matrix & A , const double v) {
 /*         destructor        */
 /*---------------------------*/
 SGTELIB::Matrix::~Matrix ( void ) {
-  //SGTELIB::rout << "Delete " << _name << "\n";
-  SGTELIB::rout.flush();
+  /*
+  #ifdef SGTELIB_DEBUG
+    SGTELIB::rout << "Delete " << _name << " of size (" << _nbCols << "," << _nbRows << ")...";
+    display(SGTELIB::rout);
+    SGTELIB::rout.flush();
+  #endif
+  */
   for ( int i = 0 ; i < _nbRows ; ++i )
-    delete [] X0[i];
-  delete [] X0;
+    delete [] X_X[i];
+  delete [] X_X; 
+  /*
+  #ifdef SGTELIB_DEBUG
+    SGTELIB::rout << "Deleted.\n";
+  #endif
+  */
 }//
 
 /*---------------------------*/
@@ -338,17 +364,17 @@ SGTELIB::Matrix::~Matrix ( void ) {
 /*---------------------------*/
 void SGTELIB::Matrix::add_row  ( const double * row ) {
 
-  double ** newX0 = new double * [_nbRows+1];
+  double ** new_X = new double * [_nbRows+1];
 
   for ( int i = 0 ; i < _nbRows ; ++i )
-    newX0[i] = X0[i];
+    new_X[i] = X_X[i];
 
-  newX0[_nbRows] = new double [_nbCols];
+  new_X[_nbRows] = new double [_nbCols];
   for ( int j = 0 ; j < _nbCols ; ++j )
-    newX0[_nbRows][j] = row[j];
+    new_X[_nbRows][j] = row[j];
   
-  delete [] X0;
-  X0 = newX0;
+  delete [] X_X;
+  X_X = new_X;
   ++_nbRows;
 }//
 
@@ -364,19 +390,19 @@ void SGTELIB::Matrix::add_rows ( const Matrix & A ) {
   int i , j;
   int new_nbRows = _nbRows + A._nbRows;
 
-  double ** newX0 = new double * [new_nbRows];
+  double ** new_X = new double * [new_nbRows];
 
   for ( i = 0 ; i < _nbRows ; ++i )
-    newX0[i] = X0[i];
+    new_X[i] = X_X[i];
 
   for ( i = _nbRows ; i < new_nbRows ; ++i ) {
-    newX0[i] = new double [_nbCols];
+    new_X[i] = new double [_nbCols];
     for ( j = 0 ; j < _nbCols ; ++j )
-      newX0[i][j] = A.X0[i-_nbRows][j];
+      new_X[i][j] = A.X_X[i-_nbRows][j];
   }
 
-  delete [] X0;
-  X0 = newX0;
+  delete [] X_X;
+  X_X = new_X;
   _nbRows = new_nbRows;
 }//
 
@@ -398,14 +424,14 @@ void SGTELIB::Matrix::add_cols ( const Matrix & A ) {
     x = new double [new_nbCols];
     // Original columns
     for ( j = 0 ; j < _nbCols ; ++j )
-      x[j] = X0[i][j];
+      x[j] = X_X[i][j];
     // Additional columns
     for ( j = _nbCols ; j < new_nbCols ; ++j )
-      x[j] = A.X0[i][j-_nbCols];
+      x[j] = A.X_X[i][j-_nbCols];
     // Remove original line
-    delete [] X0[i];
+    delete [] X_X[i];
     // Put new line
-    X0[i] = x;
+    X_X[i] = x;
   }
 
   _nbCols = new_nbCols;
@@ -419,20 +445,20 @@ void SGTELIB::Matrix::add_rows ( const int p ) {
   int i , j;
   int new_nbRows = _nbRows + p;
 
-  double ** newX0 = new double * [new_nbRows];
+  double ** new_X = new double * [new_nbRows];
 
   for ( i = 0 ; i < _nbRows ; ++i )
-    newX0[i] = X0[i];
+    new_X[i] = X_X[i];
 
   for ( i = _nbRows ; i < new_nbRows ; ++i ) {
-    newX0[i] = new double [_nbCols];
+    new_X[i] = new double [_nbCols];
     for ( j = 0 ; j < _nbCols ; ++j )
-      newX0[i][j] = 0.0;
+      new_X[i][j] = 0.0;
   }
 
-  delete [] X0;
+  delete [] X_X;
 
-  X0 = newX0;
+  X_X = new_X;
   _nbRows = new_nbRows;
 }//
 
@@ -444,16 +470,16 @@ void SGTELIB::Matrix::remove_rows ( const int p ) {
   int i;
   int new_nbRows = _nbRows - p;
 
-  double ** newX0 = new double * [new_nbRows];
+  double ** new_X = new double * [new_nbRows];
 
   for ( i = 0 ; i < new_nbRows ; ++i )
-    newX0[i] = X0[i];
+    new_X[i] = X_X[i];
 
   for ( i = new_nbRows ; i < _nbRows ; ++i )
-    delete [] X0[i];
+    delete [] X_X[i];
 
-  delete [] X0;
-  X0 = newX0;
+  delete [] X_X;
+  X_X = new_X;
   _nbRows = new_nbRows;
 }//
 
@@ -470,11 +496,11 @@ void SGTELIB::Matrix::add_cols ( const int p ) {
   for ( i = 0 ; i < _nbRows ; ++i ) {
     x = new double [new_nbCols];
     for ( j = 0 ; j < _nbCols ; ++j )
-      x[j] = X0[i][j];
+      x[j] = X_X[i][j];
     for ( j = _nbCols ; j < new_nbCols ; ++j )
       x[j] = 0.0;
-    delete [] X0[i];
-    X0[i] = x;
+    delete [] X_X[i];
+    X_X[i] = x;
   }
 
   _nbCols = new_nbCols;
@@ -499,7 +525,7 @@ bool SGTELIB::Matrix::is_sym ( void ) const {
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = i+1 ; j < _nbCols ; ++j ) {
-      if (X0[i][j]!=X0[j][i]){
+      if (X_X[i][j]!=X_X[j][i]){
         return false;
       }
     }
@@ -515,9 +541,9 @@ void SGTELIB::Matrix::set_random ( double l , double u , bool round ) {
   int i , j;
   for ( i = 0 ; i < _nbRows ; ++i ){
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] = l + (u-l) * SGTELIB::uniform_rand();
+      X_X[i][j] = l + (u-l) * SGTELIB::uniform_rand();
       if ( round )
-        X0[i][j] = SGTELIB::round ( X0[i][j] );
+        X_X[i][j] = SGTELIB::round ( X_X[i][j] );
     }
   }
 }//
@@ -529,7 +555,7 @@ void SGTELIB::Matrix::fill ( double v ) {
   int i , j;
   for ( i = 0 ; i < _nbRows ; ++i ){
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] = v;
+      X_X[i][j] = v;
     }
   }
 }//
@@ -546,7 +572,7 @@ void SGTELIB::Matrix::set ( const int i , const int j , const double d ) {
       throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::set(i,j): bad index" );
     }
   #endif
-  X0[i][j] = d;
+  X_X[i][j] = d;
 }//
 
 void SGTELIB::Matrix::set_row (const SGTELIB::Matrix & T , const int i){
@@ -556,7 +582,7 @@ void SGTELIB::Matrix::set_row (const SGTELIB::Matrix & T , const int i){
     }
   #endif
   for (int j=0 ; j<_nbCols ; j++){
-    X0[i][j] = T.get(0,j);
+    X_X[i][j] = T.get(0,j);
   }
 }//
 
@@ -567,7 +593,7 @@ void SGTELIB::Matrix::set_col (const SGTELIB::Matrix & T , const int j){
     }
   #endif
   for (int i=0 ; i<_nbRows ; i++){
-    X0[i][j] = T.get(i,0);
+    X_X[i][j] = T.get(i,0);
   }
 }//
 
@@ -578,7 +604,7 @@ void SGTELIB::Matrix::set_row (const double v , const int i){
     }
   #endif
   for (int j=0 ; j<_nbCols ; j++){
-    X0[i][j] = v;
+    X_X[i][j] = v;
   }
 }//
 
@@ -589,19 +615,19 @@ void SGTELIB::Matrix::set_col (const double v , const int j){
     }
   #endif
   for (int i=0 ; i<_nbRows ; i++){
-    X0[i][j] = v;
+    X_X[i][j] = v;
   }
 }//
 
-void SGTELIB::Matrix::permute (const int i1 , const int j1 , const int i2 , const int j2 ){
+void SGTELIB::Matrix::swap (const int i1 , const int j1 , const int i2 , const int j2 ){
   #ifdef SGTELIB_DEBUG
     if ( i1 < 0 || i1 >= _nbRows || j1 < 0 || j1 >= _nbCols || i2 < 0 || i2 >= _nbRows || j2 < 0 || j2 >= _nbCols ){
       throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::permut: bad index" );
     }
   #endif
-  double buffer = X0[i1][j1];
-  X0[i1][j1] = X0[i2][j2];
-  X0[i2][j2] = buffer;
+  double buffer = X_X[i1][j1];
+  X_X[i1][j1] = X_X[i2][j2];
+  X_X[i2][j2] = buffer;
 }//
 
 void SGTELIB::Matrix::multiply_row (const double v , const int i){
@@ -611,7 +637,7 @@ void SGTELIB::Matrix::multiply_row (const double v , const int i){
     }
   #endif
   for (int j=0 ; j<_nbCols ; j++){
-    X0[i][j] *= v;
+    X_X[i][j] *= v;
   }
 }//
 
@@ -622,7 +648,7 @@ void SGTELIB::Matrix::multiply_col (const double v , const int j){
     }
   #endif
   for (int i=0 ; i<_nbRows ; i++){
-    X0[i][j] *= v;
+    X_X[i][j] *= v;
   }
 }//
 
@@ -639,28 +665,14 @@ const double & SGTELIB::Matrix::operator [] ( int k ) const {
   if (_nbRows==1) j=k; 
   else if (_nbCols==1) i=k;
   else throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::[k]: the matrix is not a vector" );
-  return X0[i][j];
+  return X_X[i][j];
 }//
 double & SGTELIB::Matrix::operator [] ( int k ){
   int i = 0 , j = 0;
   if (_nbRows==1) j=k; 
   else if (_nbCols==1) i=k;
   else throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::[k]: the matrix is not a vector" );
-  return X0[i][j];
-}//
-
-/*---------------------------*/
-/*  access to element (i,j)  */
-/*---------------------------*/
-double SGTELIB::Matrix::get ( const int i , const int j ) const {
-  #ifdef SGTELIB_DEBUG
-    if ( i < 0 || i >= _nbRows || j < 0 || j >= _nbCols ){
-      display(SGTELIB::rout);
-      SGTELIB::rout << "Error: try to access (" << i << "," << j << ") while dim is [" << _nbRows << "," << _nbCols << "]\n";
-      throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::get(i,j): bad index" );
-    }
-  #endif
-  return X0[i][j];
+  return X_X[i][j];
 }//
 
 /*------------------------------------------*/
@@ -680,7 +692,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_row (const int i) const {
   #endif
   SGTELIB::Matrix A (_name+"(i,:)",1,_nbCols);
   for (int j=0 ; j<_nbCols ; j++){
-    A.X0[0][j] = X0[i][j];
+    A.X_X[0][j] = X_X[i][j];
   }
   return A;
 }//
@@ -695,7 +707,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_col (const int j) const {
   #endif
   SGTELIB::Matrix A (_name+"(:,j)",_nbRows,1);
   for (int i=0 ; i<_nbRows ; i++){
-    A.X0[i][0] = X0[i][j];
+    A.X_X[i][0] = X_X[i][j];
   }
   return A;
 }//
@@ -803,7 +815,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_cols (const std::list<int> & list_cols) con
 int SGTELIB::Matrix::get_nb_diff_values ( int j ) const {
   std::set<double> s;
   for ( int i = 0 ; i < _nbRows ; ++i ){
-    s.insert ( X0[i][j] );
+    s.insert ( X_X[i][j] );
   }
   return static_cast<int> ( s.size() );
 }//
@@ -816,7 +828,7 @@ void SGTELIB::Matrix::display ( std::ostream & out ) const {
   out << std::endl << _name << "=[\n";
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j )
-      out << "\t" << std::setw(10) << X0[i][j] << " ";
+      out << "\t" << std::setw(10) << X_X[i][j] << " ";
     out << ";" << std::endl;
   }
   out << "];" << std::endl;
@@ -829,15 +841,15 @@ void SGTELIB::Matrix::display_short ( std::ostream & out ) const {
   if (get_numel()<5) display(out);
   else{
     out << std::endl << _name << " ( " << _nbRows << " x " << _nbCols << " ) =\n[";
-    out << "\t" << std::setw(10) << X0[0][0] << " ";
+    out << "\t" << std::setw(10) << X_X[0][0] << " ";
     if (_nbCols>2) out << "... ";
-    out << "\t" << std::setw(10) << X0[0][_nbCols] << "\n";
+    out << "\t" << std::setw(10) << X_X[0][_nbCols] << "\n";
     if (_nbRows>2) out << "\t       ...";
     if (_nbCols>2) out << "    ";
     if (_nbRows>2) out << "\t       ...\n";
-    out << "\t" << std::setw(10) << X0[_nbRows-1][0] << " ";
+    out << "\t" << std::setw(10) << X_X[_nbRows-1][0] << " ";
     if (_nbCols>2) out << "... ";
-    out << "\t" << std::setw(10) << X0[_nbRows-1][_nbCols] << "]\n";
+    out << "\t" << std::setw(10) << X_X[_nbRows-1][_nbCols] << "]\n";
   }
 }//
 
@@ -868,7 +880,7 @@ SGTELIB::Matrix SGTELIB::Matrix::col_vector ( const double * v,
   }  
   SGTELIB::Matrix V("V",n,1);
   for (int i=0 ; i<n ; i++){
-    V.X0[i][0] = v[i];
+    V.X_X[i][0] = v[i];
   }
   return V;
 }//
@@ -883,7 +895,7 @@ SGTELIB::Matrix SGTELIB::Matrix::row_vector ( const double * v,
   }  
   SGTELIB::Matrix V("V",1,n);
   for (int i=0 ; i<n ; i++){
-    V.X0[0][i] = v[i];
+    V.X_X[0][i] = v[i];
   }
   return V;
 }//
@@ -894,14 +906,17 @@ SGTELIB::Matrix SGTELIB::Matrix::row_vector ( const double * v,
 SGTELIB::Matrix SGTELIB::Matrix::product ( const SGTELIB::Matrix & A,
                                            const SGTELIB::Matrix & B )  {
 
-  if (A.get_nb_cols()!=B.get_nb_rows()){
-    SGTELIB::rout << "A (" << A.get_name() << ") : " << A.get_nb_rows() << " , " << A.get_nb_cols() << "\n";
-    SGTELIB::rout << "B (" << B.get_name() << ") : " << B.get_nb_rows() << " , " << B.get_nb_cols() << "\n";
-    throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::product(A,B): dimension error" );
-  }  
-
-  // Init matrix
-  SGTELIB::Matrix C(A.get_name()+"*"+B.get_name(),A.get_nb_rows(),B.get_nb_cols());
+  #ifdef SGTELIB_DEBUG
+    if (A.get_nb_cols()!=B.get_nb_rows()){
+      SGTELIB::rout << "A (" << A.get_name() << ") : " << A.get_nb_rows() << " , " << A.get_nb_cols() << "\n";
+      SGTELIB::rout << "B (" << B.get_name() << ") : " << B.get_nb_rows() << " , " << B.get_nb_cols() << "\n";
+      throw SGTELIB::Exception ( __FILE__ , __LINE__ , "Matrix::product(A,B): dimension error" );
+    }  
+    // Init matrix
+    SGTELIB::Matrix C(A.get_name()+"*"+B.get_name(),A.get_nb_rows(),B.get_nb_cols());
+  #else
+    SGTELIB::Matrix C("C",A.get_nb_rows(),B.get_nb_cols());
+  #endif
 
   // Compute
   int i,j,k;
@@ -911,9 +926,11 @@ SGTELIB::Matrix SGTELIB::Matrix::product ( const SGTELIB::Matrix & A,
   //double v;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = 0;
-      for ( k = 0 ; k < nb_inter; ++k ){
-        C.X0[i][j] += A.X0[i][k]*B.X0[k][j];
+      C.X_X[i][j] = 0;
+    }
+    for ( k = 0 ; k < nb_inter; ++k ){
+      for ( j = 0 ; j < nb_cols ; ++j ){
+        C.X_X[i][j] += A.X_X[i][k]*B.X_X[k][j];
       }
     }
   }
@@ -984,7 +1001,7 @@ SGTELIB::Matrix SGTELIB::Matrix::subset_product (const SGTELIB::Matrix & A,
   for (int i=0 ; i<p ; i++){
     for (int j=0 ; j<r ; j++){
       for (int k=0 ; k<q ; k++){
-        C.X0[i][j] += A.X0[i][k]*B.X0[k][j];
+        C.X_X[i][j] += A.X_X[i][k]*B.X_X[k][j];
       } 
     }
   }
@@ -1037,8 +1054,8 @@ SGTELIB::Matrix SGTELIB::Matrix::hadamard_square ( const SGTELIB::Matrix & A )
   double a;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      a = A.X0[i][j];
-      C.X0[i][j] = a*a;
+      a = A.X_X[i][j];
+      C.X_X[i][j] = a*a;
     }
   }
   return C;
@@ -1055,7 +1072,7 @@ void SGTELIB::Matrix::hadamard_square ( void )  {
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] *= X0[i][j];
+      X_X[i][j] *= X_X[i][j];
     }
   }
 }//
@@ -1071,7 +1088,7 @@ void SGTELIB::Matrix::hadamard_inverse ( void )  {
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] = 1/X0[i][j];
+      X_X[i][j] = 1/X_X[i][j];
     }
   }
 }//
@@ -1091,7 +1108,7 @@ SGTELIB::Matrix SGTELIB::Matrix::hadamard_sqrt ( const SGTELIB::Matrix & A )  {
   int i,j;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = sqrt(fabs(A.X0[i][j]));
+      C.X_X[i][j] = sqrt(fabs(A.X_X[i][j]));
     }
   }
   return C;
@@ -1114,7 +1131,7 @@ SGTELIB::Matrix SGTELIB::Matrix::hadamard_power ( const SGTELIB::Matrix & A , co
   int i,j;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = pow(A.X0[i][j],e);
+      C.X_X[i][j] = pow(A.X_X[i][j],e);
     }
   }
   return C;
@@ -1131,7 +1148,7 @@ void SGTELIB::Matrix::hadamard_sqrt ( void )  {
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] = sqrt(fabs(X0[i][j]));
+      X_X[i][j] = sqrt(fabs(X_X[i][j]));
     }
   }
 }//
@@ -1157,9 +1174,9 @@ SGTELIB::Matrix SGTELIB::Matrix::diagA_product ( const SGTELIB::Matrix & A,
   if ( (na==ma) || (ma==n) ){
     // A is square, use the diag terms
     for ( i = 0 ; i < n ; i++ ) {
-      Aii = A.X0[i][i];
+      Aii = A.X_X[i][i];
       for ( j = 0 ; j < m ; j++ ) {
-        C.X0[i][j] = Aii*B.X0[i][j];
+        C.X_X[i][j] = Aii*B.X_X[i][j];
       }
     }
     return C;
@@ -1167,9 +1184,9 @@ SGTELIB::Matrix SGTELIB::Matrix::diagA_product ( const SGTELIB::Matrix & A,
   else if ( (na==1) && (ma==n) ){
     // A is a line vector
     for ( i = 0 ; i < n ; i++ ) {
-      Aii = A.X0[0][i];
+      Aii = A.X_X[0][i];
       for ( j = 0 ; j < m ; j++ ) {
-        C.X0[i][j] = Aii*B.X0[i][j];
+        C.X_X[i][j] = Aii*B.X_X[i][j];
       }
     }
     return C;
@@ -1177,9 +1194,9 @@ SGTELIB::Matrix SGTELIB::Matrix::diagA_product ( const SGTELIB::Matrix & A,
   else if ( (na==n) && (ma==1) ){
     // A is a col vector
     for ( i = 0 ; i < n ; i++ ) {
-      Aii = A.X0[i][0];
+      Aii = A.X_X[i][0];
       for ( j = 0 ; j < m ; j++ ) {
-        C.X0[i][j] = Aii*B.X0[i][j];
+        C.X_X[i][j] = Aii*B.X_X[i][j];
       }
     }
     return C;
@@ -1213,9 +1230,9 @@ SGTELIB::Matrix SGTELIB::Matrix::diagB_product ( const SGTELIB::Matrix & A,
   if ( (nb==mb) && (mb==n) ){
     // B is square, use the diag terms
     for ( j = 0 ; j < m ; j++ ) {
-      Bjj = B.X0[j][j];
+      Bjj = B.X_X[j][j];
       for ( i = 0 ; i < n ; i++ ) {
-        C.X0[i][j] = A.X0[i][j]*Bjj;
+        C.X_X[i][j] = A.X_X[i][j]*Bjj;
       }
     }
     return C;
@@ -1223,9 +1240,9 @@ SGTELIB::Matrix SGTELIB::Matrix::diagB_product ( const SGTELIB::Matrix & A,
   else if ( (nb==1) && (mb==m) ){
     // B is a line vector
     for ( j = 0 ; j < m ; j++ ) {
-      Bjj = B.X0[0][j];
+      Bjj = B.X_X[0][j];
       for ( i = 0 ; i < n ; i++ ) {
-        C.X0[i][j] = A.X0[i][j]*Bjj;
+        C.X_X[i][j] = A.X_X[i][j]*Bjj;
       }
     }
     return C;
@@ -1233,9 +1250,9 @@ SGTELIB::Matrix SGTELIB::Matrix::diagB_product ( const SGTELIB::Matrix & A,
   else if ( (nb==m) && (mb==1) ){
     // B is a col vector
     for ( j = 0 ; j < m ; j++ ) {
-      Bjj = B.X0[j][0];
+      Bjj = B.X_X[j][0];
       for ( i = 0 ; i < n ; i++ ) {
-        C.X0[i][j] = A.X0[i][j]*Bjj;
+        C.X_X[i][j] = A.X_X[i][j]*Bjj;
       }
     }
     return C;
@@ -1270,9 +1287,9 @@ SGTELIB::Matrix SGTELIB::Matrix::transposeA_product ( const SGTELIB::Matrix & A,
   //double v;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = 0;
+      C.X_X[i][j] = 0;
       for ( k = 0 ; k < nb_inter; ++k ){
-        C.X0[i][j] += A.X0[k][i]*B.X0[k][j];
+        C.X_X[i][j] += A.X_X[k][i]*B.X_X[k][j];
       }
     }
   }
@@ -1343,7 +1360,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_matrix_dPiPZs ( const SGTELIB::Matrix & Ai,
   int i,j;
   for ( i = 0 ; i < p ; i++ ) {
     v = 0;
-    for ( j = 0 ; j < q ; j++) v += HAi.X0[i][j]*H.X0[i][j];
+    for ( j = 0 ; j < q ; j++) v += HAi.X_X[i][j]*H.X_X[i][j];
     v = 1.0/( 1.0 - v );
     dPiPZs.multiply_row ( v , i );
   }
@@ -1371,7 +1388,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_matrix_dPiPZs ( const SGTELIB::Matrix & Ai,
   int i,j;
   for ( i = 0 ; i < p ; i++ ) {
     v = 0;
-    for ( j = 0 ; j < q ; j++) v += HAi.X0[i][j]*H.X0[i][j];
+    for ( j = 0 ; j < q ; j++) v += HAi.X_X[i][j]*H.X_X[i][j];
     v = 1.0/( 1.0 - v );
     dPiPZs.multiply_row ( v , i );
   }
@@ -1425,7 +1442,7 @@ SGTELIB::Matrix SGTELIB::Matrix::add ( const SGTELIB::Matrix & A,
   int i,j;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = A.X0[i][j]+B.X0[i][j];
+      C.X_X[i][j] = A.X_X[i][j]+B.X_X[i][j];
     }
   }
   return C;
@@ -1447,7 +1464,7 @@ void SGTELIB::Matrix::add ( const SGTELIB::Matrix & B ) {
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] += B.get(i,j);
+      X_X[i][j] += B.get(i,j);
     }
   }
 }//
@@ -1468,12 +1485,12 @@ SGTELIB::Matrix SGTELIB::Matrix::add_fill ( const SGTELIB::Matrix & A,
   int i,j;
   for ( i = 0 ; i < A.get_nb_rows() ; ++i ) {
     for ( j = 0 ; j < A.get_nb_cols() ; ++j ){
-      C.X0[i][j] = A.X0[i][j];
+      C.X_X[i][j] = A.X_X[i][j];
     }
   }
   for ( i = 0 ; i < B.get_nb_rows() ; ++i ) {
     for ( j = 0 ; j < B.get_nb_cols() ; ++j ){
-      C.X0[i][j] += B.X0[i][j];
+      C.X_X[i][j] += B.X_X[i][j];
     }
   }
   return C;
@@ -1526,7 +1543,7 @@ void SGTELIB::Matrix::sub ( const SGTELIB::Matrix & B ) {
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      X0[i][j] -= B.X0[i][j];
+      X_X[i][j] -= B.X_X[i][j];
     }
   }
 }//
@@ -1574,10 +1591,10 @@ SGTELIB::Matrix SGTELIB::Matrix::random_permutation_matrix ( const int n ) {
   for (int i=0; i<n; ++i) v.push_back(i); // 1 2 3 4 5 6 7 8 9
 
   // shuffle
-  std::random_shuffle ( v.begin(), v.end(), SGTELIB::Matrix::myrandom );
-
+  SGTELIB::random_shuffle ( v.begin(), v.end(), SGTELIB::Matrix::myrandom );    //zhenghua
+ 
   // Fill matrix
-  for (int i=0; i<n; ++i) perm.set(i,v.at(i),1.0);
+  for (int i=0; i<n; ++i) perm.set(i,v[i],1.0);
 
   return perm;
 }//
@@ -1605,9 +1622,9 @@ SGTELIB::Matrix SGTELIB::Matrix::rank ( void ) const {
     for (i=0 ; i<m ; i++){
       dmin = +INF;
       for (j=0 ; j<m ; j++){
-        if (D.X0[0][j]<dmin){
+        if (D.X_X[0][j]<dmin){
           jmin = j;
-          dmin = D.X0[0][j];
+          dmin = D.X_X[0][j];
         }
       }
       R.set(0,jmin,double(i));
@@ -1641,7 +1658,7 @@ double SGTELIB::Matrix::rmse ( void ) const{
   double xij;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      xij = X0[i][j];
+      xij = X_X[i][j];
       v += xij*xij;
     }
   }
@@ -1660,7 +1677,7 @@ double SGTELIB::Matrix::normsquare ( void ) const{
   double xij;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      xij = X0[i][j];
+      xij = X_X[i][j];
       v += xij*xij;
     }
   }
@@ -1679,21 +1696,55 @@ void SGTELIB::Matrix::normalize_cols ( void ){
     d = 0;
 
     for ( i = 0 ; i < _nbRows ; ++i )
-      d += X0[i][j];
+      d += X_X[i][j];
 
     if (d==0){
       for ( i = 0 ; i < _nbRows ; ++i )
-        X0[i][j] = 1/_nbRows;
+        X_X[i][j] = 1/_nbRows;
     }
     else{
       for ( i = 0 ; i < _nbRows ; ++i )
-        X0[i][j] /= d;
+        X_X[i][j] /= d;
     }
   }
 }//
 
 
 
+/*------------------------------------------*/
+/*  compute the norm of each column         */
+/*------------------------------------------*/
+SGTELIB::Matrix SGTELIB::Matrix::col_norm ( const norm_t nt ) const {
+  
+  SGTELIB::Matrix N = SGTELIB::Matrix("Norm",1,_nbCols);
+
+  int i,j;
+  for ( j = 0 ; j < _nbCols ; ++j ){
+    double v = 0;
+    switch (nt){
+      case SGTELIB::NORM_0:
+        for (i=0;i<_nbRows;++i) v += double(fabs(X_X[i][j])<EPSILON);
+        v /= double(_nbCols);
+        break;
+      case SGTELIB::NORM_1:
+        for (i=0;i<_nbRows;++i) v += fabs(X_X[i][j]);
+        v /= double(_nbCols);
+        break;
+      case SGTELIB::NORM_2:
+        for (i=0;i<_nbRows;++i) v += X_X[i][j]*X_X[i][j];
+        v = sqrt(v/double(_nbCols));
+        break;
+      case SGTELIB::NORM_INF:
+        for (i=0;i<_nbRows;++i) v = std::max(v,fabs(X_X[i][j]));
+        break;
+    }
+    N.set(0,j,v);
+  }
+  return N;
+}//
+
+
+
 /*---------------------------*/
 /*        square norm        */
 /*---------------------------*/
@@ -1711,7 +1762,7 @@ double SGTELIB::Matrix::sum ( void ) const{
   int i,j;
   for ( i = 0 ; i < _nbRows ; ++i ) {
     for ( j = 0 ; j < _nbCols ; ++j ){
-      v += X0[i][j];
+      v += X_X[i][j];
     }
   }
   return v;
@@ -1733,9 +1784,9 @@ SGTELIB::Matrix SGTELIB::Matrix::sum ( const int direction ) const{
     for ( j = 0 ; j < _nbCols ; ++j ){
       v = 0;
       for ( i = 0 ; i < _nbRows ; ++i ) {
-        v += X0[i][j];
+        v += X_X[i][j];
       }
-      S.X0[0][j] = v;
+      S.X_X[0][j] = v;
     }
     return S;
   }
@@ -1744,9 +1795,9 @@ SGTELIB::Matrix SGTELIB::Matrix::sum ( const int direction ) const{
     for ( i = 0 ; i < _nbRows ; ++i ) {
       v = 0;
       for ( j = 0 ; j < _nbCols ; ++j ){
-        v += X0[i][j];
+        v += X_X[i][j];
       }
-      S.X0[i][0] = v;
+      S.X_X[i][0] = v;
     }
     return S;
   }
@@ -1776,7 +1827,7 @@ int SGTELIB::Matrix::count ( void ) const{
   const int nb_cols = get_nb_cols();
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      v += (fabs(X0[i][j])>EPSILON)? 1:0 ;
+      v += (fabs(X_X[i][j])>EPSILON)? 1:0 ;
     }
   }
   return v;
@@ -1827,7 +1878,7 @@ SGTELIB::Matrix SGTELIB::Matrix::conjugate_solve ( const SGTELIB::Matrix & A ,
   while (iter < 100){
     Ap = A*p;
     pAp = 0;
-    for (int i=0 ; i<n ; i++) pAp += p.X0[i][0]*Ap.X0[i][0];
+    for (int i=0 ; i<n ; i++) pAp += p.X_X[i][0]*Ap.X_X[i][0];
     alpha = rr/pAp;
     x = x+alpha*p;
     rr_old = rr;
@@ -1863,11 +1914,11 @@ SGTELIB::Matrix SGTELIB::Matrix::cholesky ( void ) const {
     for (j = 0; j < (i+1); j++) {
       s = 0;
       for (k = 0; k < j; k++){
-        s += L.X0[i][k] * L.X0[j][k];
+        s += L.X_X[i][k] * L.X_X[j][k];
       }
-      L.X0[i][j] = (i == j) ?
-         sqrt(X0[i][i] - s) :
-         (1.0 / L.X0[j][j] * (X0[i][j] - s));
+      L.X_X[i][j] = (i == j) ?
+         sqrt(X_X[i][i] - s) :
+         (1.0 / L.X_X[j][j] * (X_X[i][j] - s));
     }
   }
   return L;
@@ -1895,17 +1946,17 @@ SGTELIB::Matrix SGTELIB::Matrix::cholesky_inverse ( double * det ) const {
   int i,j,k,kmin;
   for (i=0 ; i<n ; i++){
     for (j=0 ; j<n ; j++){
-      A.X0[i][j] = 0;
+      A.X_X[i][j] = 0;
       kmin = std::max(i,j);
       for (k=kmin ; k<n ; k++){
-          A.X0[i][j] += Li.X0[k][i]*Li.X0[k][j];
+          A.X_X[i][j] += Li.X_X[k][i]*Li.X_X[k][j];
       }
     }
   }
 
   if (det){
     double v = 1;
-    for (i=0 ; i<n ; i++) v *= L.X0[i][i];
+    for (i=0 ; i<n ; i++) v *= L.X_X[i][i];
     v *= v;
     if ( crs_isnan(v)) v=+INF;
     *det = v;
@@ -1937,9 +1988,9 @@ SGTELIB::Matrix SGTELIB::Matrix::triu_solve( const SGTELIB::Matrix & U ,
 
   for (int i=n-1 ; i>=0 ; i--){
     for (int j=i+1 ; j<n ; j++){
-      x.X0[i][0] -= U.X0[i][j]*x.X0[j][0];
+      x.X_X[i][0] -= U.X_X[i][j]*x.X_X[j][0];
     }
-    x.X0[i][0] /= U.X0[i][i];
+    x.X_X[i][0] /= U.X_X[i][i];
   }
 
   return x;
@@ -1985,9 +2036,9 @@ SGTELIB::Matrix SGTELIB::Matrix::tril_solve( const SGTELIB::Matrix & L ,
 
   for (int i=0 ; i<n ; i++){
     for (int j=0 ; j<i ; j++){
-      x.X0[i][0] -= L.X0[i][j]*x.X0[j][0];
+      x.X_X[i][0] -= L.X_X[i][j]*x.X_X[j][0];
     }
-    x.X0[i][0] /= L.X0[i][i];
+    x.X_X[i][0] /= L.X_X[i][i];
   }
 
   return x;
@@ -2044,7 +2095,7 @@ SGTELIB::Matrix SGTELIB::Matrix::transpose ( void ) const{
   SGTELIB::Matrix A (_name+"'",_nbCols,_nbRows); 
   for (int i=0 ; i<_nbCols ; i++){
     for (int j=0 ; j<_nbRows ; j++){
-      A.set(i,j,X0[j][i]);
+      A.set(i,j,X_X[j][i]);
     }
   }
   return A;
@@ -2058,7 +2109,7 @@ SGTELIB::Matrix SGTELIB::Matrix::diag ( void ) const{
   SGTELIB::Matrix A;
   if (_nbCols==_nbRows){
     A = SGTELIB::Matrix("A",_nbRows,1);   
-    for (int i=0 ; i<_nbCols ; i++) A.set(i,0,X0[i][i]);
+    for (int i=0 ; i<_nbCols ; i++) A.set(i,0,X_X[i][i]);
   }
   else if ( (_nbCols==1) || (_nbRows==1) ){
     const int n=std::max(_nbCols,_nbRows);
@@ -2183,7 +2234,7 @@ bool SGTELIB::Matrix::SVD_decomposition ( std::string & error_msg ,
   // copy the current matrix into U:
   for ( i = 0 ; i < nbRows ; ++i )
     for ( j = 0 ; j < nbCols ; ++j )
-      U[i][j] = X0[i][j];
+      U[i][j] = X_X[i][j];
 
   // Householder reduction to bidiagonal form:
   for ( i = 0 ; i < nbCols ; ++i ) {
@@ -2426,7 +2477,7 @@ bool SGTELIB::Matrix::has_nan ( void ) const {
   int i , j;
   for ( i = 0 ; i < _nbRows ; ++i ){
     for ( j = 0 ; j < _nbCols ; ++j ){
-      if ( crs_isnan(X0[i][j])){
+      if ( crs_isnan(X_X[i][j])){
         return true;
       }
     }
@@ -2441,7 +2492,7 @@ bool SGTELIB::Matrix::has_inf ( void ) const {
   int i , j;
   for ( i = 0 ; i < _nbRows ; ++i ){
     for ( j = 0 ; j < _nbCols ; ++j ){
-      if ( crs_isinf(X0[i][j])){
+      if ( crs_isinf(X_X[i][j])){
         return true;
       }
     }
@@ -2456,8 +2507,8 @@ void SGTELIB::Matrix::replace_nan ( double d ) {
   int i , j;
   for ( i = 0 ; i < _nbRows ; ++i ){
     for ( j = 0 ; j < _nbCols ; ++j ){
-      if ( crs_isnan(X0[i][j])){
-        X0[i][j] = d;
+      if ( crs_isnan(X_X[i][j])){
+        X_X[i][j] = d;
       }
     }
   }
@@ -2475,8 +2526,8 @@ int SGTELIB::Matrix::get_max_index (void ) {
   //   3 6
   for ( j = 0 ; j < _nbCols ; ++j ){
     for ( i = 0 ; i < _nbRows ; ++i ){
-      if (X0[i][j] > vmax){
-        vmax = X0[i][j];
+      if (X_X[i][j] > vmax){
+        vmax = X_X[i][j];
         kmax = k;
       }
       k++;
@@ -2493,7 +2544,7 @@ double SGTELIB::Matrix::min (void) {
   int i,j;
   for ( j = 0 ; j < _nbCols ; ++j ){
     for ( i = 0 ; i < _nbRows ; ++i ){
-      d = std::min(d,X0[i][j]);
+      d = std::min(d,X_X[i][j]);
     }
   }
   return d;
@@ -2504,7 +2555,7 @@ double SGTELIB::Matrix::max (void) {
   int i,j;
   for ( j = 0 ; j < _nbCols ; ++j ){
     for ( i = 0 ; i < _nbRows ; ++i ){
-      d = std::max(d,X0[i][j]);
+      d = std::max(d,X_X[i][j]);
     }
   }
   return d;
@@ -2532,7 +2583,7 @@ SGTELIB::Matrix SGTELIB::Matrix::max ( const SGTELIB::Matrix & A ,
   int i,j;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = std::max( A.X0[i][j] , B.X0[i][j] );
+      C.X_X[i][j] = std::max( A.X_X[i][j] , B.X_X[i][j] );
     }
   }
   return C;
@@ -2557,7 +2608,7 @@ SGTELIB::Matrix SGTELIB::Matrix::min ( const SGTELIB::Matrix & A ,
   int i,j;
   for ( i = 0 ; i < nb_rows ; ++i ) {
     for ( j = 0 ; j < nb_cols ; ++j ){
-      C.X0[i][j] = std::min( A.X0[i][j] , B.X0[i][j] );
+      C.X_X[i][j] = std::min( A.X_X[i][j] , B.X_X[i][j] );
     }
   }
   return C;
@@ -2575,8 +2626,8 @@ int SGTELIB::Matrix::get_min_index (void ) {
   //   3 6
   for ( j = 0 ; j < _nbCols ; ++j ){
     for ( i = 0 ; i < _nbRows ; ++i ){
-      if (X0[i][j] < vmin){
-        vmin = X0[i][j];
+      if (X_X[i][j] < vmin){
+        vmin = X_X[i][j];
         kmin = k;
       }
       k++;
@@ -2592,8 +2643,8 @@ int SGTELIB::Matrix::get_min_index_row ( const int i ) {
   int j, jmin=0;
   double vmin = +SGTELIB::INF;
   for ( j = 0 ; j < _nbCols ; ++j ){
-    if (X0[i][j] < vmin){
-      vmin = X0[i][j];
+    if (X_X[i][j] < vmin){
+      vmin = X_X[i][j];
       jmin = j;
     }
   }
@@ -2607,8 +2658,8 @@ int SGTELIB::Matrix::get_min_index_col ( const int j ) {
   int i, imin=0;
   double vmin = +SGTELIB::INF;
   for ( i = 0 ; i < _nbRows ; ++i ){
-    if (X0[i][j] < vmin){
-      vmin = X0[i][j];
+    if (X_X[i][j] < vmin){
+      vmin = X_X[i][j];
       imin = i;
     }
   }
@@ -2636,10 +2687,10 @@ SGTELIB::Matrix SGTELIB::Matrix::get_distances_norm2 ( const SGTELIB::Matrix & A
       // Distance between the point ia of the cache and the point ib of the matrix XXs
       v = 0;
       for (j=0 ; j < n ; j++){
-        d = A.X0[ia][j]-B.X0[ib][j];
+        d = A.X_X[ia][j]-B.X_X[ib][j];
         v += d*d;
       }
-      D.X0[ia][ib] = sqrt(v);
+      D.X_X[ia][ib] = sqrt(v);
     }
   }
   return D;
@@ -2666,9 +2717,9 @@ SGTELIB::Matrix SGTELIB::Matrix::get_distances_norm1 ( const SGTELIB::Matrix & A
       // Distance between the point ia of the cache and the point ib of the matrix XXs
       v = 0;
       for (j=0 ; j < n ; j++){
-        v += fabs(A.X0[ia][j]-B.X0[ib][j]);
+        v += fabs(A.X_X[ia][j]-B.X_X[ib][j]);
       }
-      D.X0[ia][ib] = v;
+      D.X_X[ia][ib] = v;
     }
   }
   return D;
@@ -2695,9 +2746,9 @@ SGTELIB::Matrix SGTELIB::Matrix::get_distances_norminf ( const SGTELIB::Matrix &
       // Distance between the point ia of the cache and the point ib of the matrix XXs
       v = 0;
       for (j=0 ; j < n ; j++){
-        v = std::max( v , fabs(A.X0[ia][j]-B.X0[ib][j]) );
+        v = std::max( v , fabs(A.X_X[ia][j]-B.X_X[ib][j]) );
       }
-      D.X0[ia][ib] = v;
+      D.X_X[ia][ib] = v;
     }
   }
   return D;
@@ -2725,7 +2776,7 @@ int SGTELIB::Matrix::find_row (SGTELIB::Matrix & R){
   for (i=0 ; i<_nbRows ; i++){
     diff = false;    
     for (j=0 ; j<_nbCols ; j++){
-      if (X0[i][j]!=R.X0[0][j]){
+      if (X_X[i][j]!=R.X_X[0][j]){
         diff = true;
         break;
       }
@@ -2770,7 +2821,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_poll_directions ( const SGTELIB::Matrix sca
     SGTELIB::Matrix v("v",1,N);
     for (j=0 ; j<N ; j++){
       if (domain[j]==SGTELIB::PARAM_DOMAIN_CONTINUOUS){
-        v.X0[0][j] = SGTELIB::quick_norm_rand();
+        v.X_X[0][j] = SGTELIB::quick_norm_rand();
       }
     }
 
@@ -2781,7 +2832,7 @@ SGTELIB::Matrix SGTELIB::Matrix::get_poll_directions ( const SGTELIB::Matrix sca
     for (i=0 ; i<N ; i++){
       if (domain[i]==SGTELIB::PARAM_DOMAIN_CONTINUOUS){
         for (j=0 ; j<N ; j++){
-          D.X0[i][j] = double(i==j)-2*v[i]*v[j];
+          D.X_X[i][j] = double(i==j)-2*v[i]*v[j];
         }
       }
     }
@@ -2798,29 +2849,29 @@ SGTELIB::Matrix SGTELIB::Matrix::get_poll_directions ( const SGTELIB::Matrix sca
     if (domain[i]!=SGTELIB::PARAM_DOMAIN_CONTINUOUS){
       for (j=0 ; j<N ; j++){
         if (domain[j]==SGTELIB::PARAM_DOMAIN_CONTINUOUS){
-          D.X0[i][j] = 2*uniform_rand()-1;
+          D.X_X[i][j] = 2*uniform_rand()-1;
         }
       }
     }
 
     // Find max asb
     d = 0;
-    for (j=0 ; j<N ; j++) d = std::max( d , fabs(D.X0[i][j]) );
+    for (j=0 ; j<N ; j++) d = std::max( d , fabs(D.X_X[i][j]) );
 
     // Scale continuous dimensions
     for (j=0 ; j<N ; j++){
       if (domain[j]==SGTELIB::PARAM_DOMAIN_CONTINUOUS){
-        D.X0[i][j] = scaling[j]*msize*SGTELIB::rceil(rho*D.X0[i][j]/d);
+        D.X_X[i][j] = scaling[j]*msize*SGTELIB::rceil(rho*D.X_X[i][j]/d);
       }
     }
 
     // Add extended POLL for discrete values
     if ( (domain[i]==SGTELIB::PARAM_DOMAIN_INTEGER) ||
          (domain[i]==SGTELIB::PARAM_DOMAIN_BOOL) ){
-      D.X0[i][i] = (i%2==0)?-1:+1;
+      D.X_X[i][i] = (i%2==0)?-1:+1;
     }
     else if (domain[i]==SGTELIB::PARAM_DOMAIN_CAT){
-      D.X0[i][i] = SGTELIB::rceil(uniform_rand()*scaling[i]);
+      D.X_X[i][i] = SGTELIB::rceil(uniform_rand()*scaling[i]);
     }
 
   }
@@ -2856,9 +2907,9 @@ SGTELIB::Matrix SGTELIB::Matrix::get_poll_directions ( const SGTELIB::Matrix sca
 void SGTELIB::Matrix::swap_rows(const int i1 , const int i2){
   double buffer;
   for (int j=0 ; j<_nbCols ; j++){
-    buffer    = X0[i1][j];
-    X0[i1][j] = X0[i2][j];
-    X0[i2][j] = buffer;
+    buffer    = X_X[i1][j];
+    X_X[i1][j] = X_X[i2][j];
+    X_X[i2][j] = buffer;
   }
 }//
 
@@ -2888,7 +2939,7 @@ SGTELIB::Matrix SGTELIB::Matrix::lu_inverse ( double * det ) const{
     // Find pivot
     pivot_max = -1;
     for (i=k ; i<N ; i++){
-      pivot = A.X0[k][i];
+      pivot = A.X_X[k][i];
       if (pivot<0) pivot*=-1;
       if (pivot>pivot_max){
         ip = i;
@@ -2904,15 +2955,15 @@ SGTELIB::Matrix SGTELIB::Matrix::lu_inverse ( double * det ) const{
 
     // Gaussian elimination
     for (j=k+1 ; j<N ; j++){
-      pivot = A.X0[j][k]/A.X0[k][k];
-      A.X0[j][k] = pivot;
-      for (i=k+1 ; i<N ; i++) A.X0[j][i] -= pivot*A.X0[k][i];
+      pivot = A.X_X[j][k]/A.X_X[k][k];
+      A.X_X[j][k] = pivot;
+      for (i=k+1 ; i<N ; i++) A.X_X[j][i] -= pivot*A.X_X[k][i];
     }
   }
 
   // Construct the whole matrix P (under the name Ai)
   SGTELIB::Matrix Ai ("Ai",N,N);
-  for (i=0 ; i<N; i++) Ai.X0[i][P[i]] = 1;
+  for (i=0 ; i<N; i++) Ai.X_X[i][P[i]] = 1;
 
 
 
@@ -2920,7 +2971,7 @@ SGTELIB::Matrix SGTELIB::Matrix::lu_inverse ( double * det ) const{
     // Compute the determinant of the matrix that is inverted
     double v = 1;
     // Compute the determinant of U
-    for (i=0 ; i<N ; i++) v *= A.X0[i][i];
+    for (i=0 ; i<N ; i++) v *= A.X_X[i][i];
     // Comput ethe determinant of P
     i = 0;
     while (i<N){
@@ -2943,16 +2994,16 @@ SGTELIB::Matrix SGTELIB::Matrix::lu_inverse ( double * det ) const{
     // Tri-L solve
     for (i=0 ; i<N ; i++){
       for (j=0 ; j<i ; j++){
-        y.X0[i][0] -= A.X0[i][j]*y.X0[j][0];
+        y.X_X[i][0] -= A.X_X[i][j]*y.X_X[j][0];
       }
     }
 
     // Tri-U solve
     for (int i=N-1 ; i>=0 ; i--){
       for (int j=i+1 ; j<N ; j++){
-        y.X0[i][0] -= A.X0[i][j]*y.X0[j][0];
+        y.X_X[i][0] -= A.X_X[i][j]*y.X_X[j][0];
       }
-      y.X0[i][0] /= A.X0[i][i];
+      y.X_X[i][0] /= A.X_X[i][i];
     }
 
     Ai.set_col(y,k);
